//
//
//  Nucleo F767
//
//
//
//
//
unit Unit_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  //
  Sockets, ScktComp;

type
  TSysTick         = Record
                      Start_  : Int64;
                      End_    : Int64;
                      Freq    : Int64;
                     End;
  TTickAct         = (taStart,
                      taStop);
  //
  TForm_Main = class(TForm)
    Memo_Snd: TMemo;
    Ed_Server: TEdit;
    Btn_UDP_Test: TButton;
    Btn_TCP_Test: TButton;
    Lbl_HostName: TLabel;
    procedure Btn_UDP_TestClick(Sender: TObject);
    procedure Btn_TCP_TestClick(Sender: TObject);
  private
    { Private declarations }
  public
    Tick : TSysTick;
    //
    Procedure Dbg( const Msg : String );
  end;

var
  Form_Main: TForm_Main;

implementation

{$R *.dfm}



//
Function  GetTick_ms (Var Tick : TSysTick; TickAct : TTickAct ) : Single;
 Var
  dTick : Int64;
 begin
  //
  Result := 0;
  //
  Case TickAct of
   taStart : begin
              QueryPerformanceFrequency(Tick.Freq  );
              QueryPerformanceCounter  (Tick.Start_);
             end;
   taStop  : begin
              QueryPerformanceCounter  (Tick.End_  );
              dTick := Tick.End_ - Tick.Start_;
              Case dTick <= 0 of
               True  : Result := 0;
               False : begin
                        If Tick.Freq > 0 then
                             Result := (dTick / Tick.Freq) * 1000 // ms
                        else Result := 0;
                       end;
              End;
             end;
  End;
 end;

Procedure TForm_Main.Dbg( const Msg : String );
 begin
  Memo_Snd.Lines.Add(Msg);
 end;


//-----------------------------------------------------------------------------
// TCP Test
//-----------------------------------------------------------------------------

procedure TForm_Main.Btn_TCP_TestClick(Sender: TObject);
 Var
  Skt    : TClientSocket;
  sBuf   : Array[0..1460-1] of Byte;
  rBuf   : Array[0..1460-1] of Byte;
  Loop   : Integer;
  i      : Integer;
  rBytes : Integer;
  Speed  : Single;
  ID     : DWord;
begin
 //
 FillChar(sBuf,SizeOf(sBuf),$00);
 Loop    := 1000;
 //
 Skt := TClientSocket.Create(Self);
 Skt.ClientType := ctBlocking;
 Skt.Host   := Ed_Server.Text;
 Skt.Port   := 80;
 Skt.Active := True;
 //
 GetTick_ms(Tick,taStart);
 For i := 0 to Loop-1 do
  begin
   ID := i;
   Move(ID,sBuf,4);
   If i = Loop-1 then sBuf[8] := $FF;
   Skt.Socket.SendBuf( sBuf, SizeOf(sBuf) );
   If i = Loop-1 then
    begin
     Repeat
      rBytes := Skt.Socket.ReceiveLength;
      If rBytes > 0 then Skt.Socket.ReceiveBuf(rBuf,rBytes);
     Until (rBytes >= 10);
     Dbg('Last Packet');
    end;
   //
   If (i mod 100) = 0 then Dbg( 'ID:' + IntToStr(i) + ' / ' + IntToStr(rBytes) + 'Bytes');
  end;
 //
 Speed := (Loop*Sizeof(sBuf) / (GetTick_ms(Tick,taStop) * 0.001) ) / (1024*1024);
 Dbg( Format('%4.1f',[Speed]) + 'MB/s' );
 //
 Skt.Active := False;
 Skt.Free;
end;

//-----------------------------------------------------------------------------
// UDP Test
//-----------------------------------------------------------------------------

// DatLen <= cMax_TCP_Payload (1472)
// Sample Packet  :  ID(4) iID(4) Ack(1)   / Ack = $FF -> Reply
procedure TForm_Main.Btn_UDP_TestClick(Sender: TObject);
 Var
  Skt    : TUDPSocket;
  sBuf   : Array[0..1472-1] of Byte;
  rBuf   : Array[0..1472-1] of Byte;
  Loop   : Integer;
  i      : Integer;
  sBytes : Integer;
  rBytes : Integer;
  Speed  : Single;
  ID     : DWord;
begin
 //
 FillChar(sBuf,SizeOf(sBuf),$00);
 sBuf[0] := 0;
 Loop    := 1000;
 //
 Skt := TUDPSocket.Create(Self);
 Skt.BlockMode  := bmBlocking;
 Skt.RemoteHost := Ed_Server.Text;
 Skt.RemotePort := '1234';
 Skt.Active     := True;
 //
 GetTick_ms(Tick,taStart);
 For i := 0 to Loop-1 do
  begin
   ID := i;
   Move(ID,sBuf,4);
   If i = Loop-1 then sBuf[8] := $FF;
   sBytes := Skt.SendBuf( sBuf, SizeOf(sBuf) );
   If i = Loop-1 then
    begin
     rBytes := Skt.ReceiveBuf(rBuf,SizeOf(rBuf),0);
     Dbg('Last Packet');
    end;
   //If rBytes > 0 then Skt.ReceiveBuf(rBuf,rBytes);
   //
   If (i mod 100) = 0 then Dbg( 'ID:' + IntToStr(i) + ' / ' + IntToStr(rBytes) +'Bytes');
  end;
 //
 Speed := (Loop*Sizeof(sBuf) / (GetTick_ms(Tick,taStop) * 0.001) ) / (1024*1024);
 Dbg( Format('%4.1f',[Speed]) + 'MB/s' );
 //
 Skt.Active := False;
 Skt.Free;
end;

end.
